//+ref=Android.UsbSerial.dll
//+ref=Mono.Android.dll
using System;
using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
using Android.Hardware.Usb;
using Hoho.Android.UsbSerial.Driver;
using Hoho.Android.UsbSerial.Extensions;
using Hoho.Android.UsbSerial.Util;

namespace CSharp_Shell
{

    public static class Program 
    {
        public static void Main() 
        {
			var drivers = FindAllDriversAsync(UsbManagerExtensions.UsbManager).Result;
			if (drivers == null || drivers.Count == 0)
			{
				throw new Exception("No driver found or no connected devices");
			}
			var port = drivers[0].Ports.First(); //Get first recognized driver/device and port.
			
			//Create serial IO manager. Paramaters: int baudRate, int dataBits, StopBits stopBits, Parity parity
			var sio = UsbManagerExtensions.CreateSerialInputOutput(port, 115200, 8, StopBits.One, Parity.None).Result;
			sio.DataReceived += (sender, e) => Console.WriteLine(e.Data);
			sio.Open(UsbManagerExtensions.UsbManager);
        }
		
		
		public static async Task<IList<IUsbSerialDriver>> FindAllDriversAsync(UsbManager usbManager)
        {
            // using the default probe table
            // return UsbSerialProber.DefaultProber.FindAllDriversAsync (usbManager);

            // adding a custom driver to the default probe table
            var table = UsbSerialProber.DefaultProbeTable;
            table.AddProduct(0x1b4f, 0x0008, typeof(CdcAcmSerialDriver)); // IOIO OTG
            table.AddProduct(0x09D8, 0x0420, typeof(CdcAcmSerialDriver)); // Elatec TWN4

            var prober = new UsbSerialProber(table);
            return await prober.FindAllDriversAsync(usbManager);
        }
    }
}